﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using Infragistics.UltraChart.Shared.Styles;
using Infragistics.UltraChart.Resources.Appearance;
using Infragistics.UltraChart.Core.Layers;

namespace WDC_BarChart
{
    #region Form1
    public partial class Form1 : Form
    {
        #region Constructor
        public Form1()
        {
            InitializeComponent();

        }
        #endregion // Constructor

        #region Form Load
        private void Form_Load(object sender, EventArgs e)
        {
            List<Data> columnData = this.CreateColumnData();
            List<Data> lineData = this.CreateLineData();

            this.ultraChart2.ChartType = ChartType.Composite;
            ChartArea myChartArea = new ChartArea();
            this.ultraChart2.CompositeChart.ChartAreas.Add(myChartArea);

            // set chart column x-axis
            AxisItem xAxisColumn = new AxisItem(ultraChart2, AxisNumber.X_Axis);
            xAxisColumn.DataType = AxisDataType.String;
            // Remove the grid lines from the XAxis.
            xAxisColumn.MajorGridLines.Visible = false;
            xAxisColumn.MinorGridLines.Visible = false;
            myChartArea.Axes.Add(xAxisColumn);

            // set chart line x-axis
            AxisItem xAxisLine = new AxisItem(ultraChart2, AxisNumber.X_Axis);
            xAxisLine.DataType = AxisDataType.String;
            xAxisLine.SetLabelAxisType = SetLabelAxisType.ContinuousData;
            // Remove the grid lines from the XAxis.
            xAxisLine.MajorGridLines.Visible = false;
            xAxisLine.MinorGridLines.Visible = false;
            myChartArea.Axes.Add(xAxisLine);

            //the layers will share the numeric y axis.
            AxisItem yAxis = new AxisItem(ultraChart2, AxisNumber.Y_Axis);
            yAxis.DataType = AxisDataType.Numeric;
            yAxis.Labels.ItemFormat = AxisItemLabelFormat.DataValue;
            yAxis.LineThickness = 1;
            yAxis.TickmarkStyle = AxisTickStyle.Smart;
            // You need to set the range type to Custom in order to be able to set the Chart range.
            yAxis.RangeType = AxisRangeType.Custom;
            yAxis.RangeMin = 30;
            yAxis.RangeMax = 2000;
            // Remove the grid lines from the YAxis.
            yAxis.MajorGridLines.Visible = false;
            yAxis.MinorGridLines.Visible = false;
            // Set the space to the right of the column.
            yAxis.Extent = 40;

            myChartArea.Axes.Add(yAxis);
            //set columnLayer
            int currentYear = DateTime.Now.Year;
            NumericSeries Series = new NumericSeries();

            ChartLayerAppearance columnLayer = new ChartLayerAppearance();
            columnLayer.AxisX = xAxisColumn;
            columnLayer.AxisY = yAxis;
            columnLayer.ChartArea = myChartArea;
            columnLayer.ChartType = ChartType.StackColumnChart;
            columnLayer.LegendItem = LegendItemType.Point;
            // No need to use the FillSceneGraph event for this.
            ((ColumnChartAppearance)columnLayer.ChartTypeAppearance).ColumnSpacing = 1;

            ChartLayerAppearance lineLayer = new ChartLayerAppearance();
            lineLayer.AxisX = xAxisLine;
            lineLayer.AxisY = yAxis;
            lineLayer.ChartArea = myChartArea;
            lineLayer.ChartType = ChartType.LineChart;
            // lineLayer.SwapRowsAndColumns = true;
            lineLayer.LegendItem = LegendItemType.Auto;

            //add all column series to column layer
            foreach (var property in typeof(Data).GetProperties())
            {
                if (property.Name != "ProductName")
                {
                    Series = this.BuildColumnSeries(CreateColumnData(), property.Name, currentYear.ToString());
                    currentYear = currentYear + 1;
                    columnLayer.Series.Add(Series);
                }
            }

            foreach (var item in lineData)
            {
                Series = this.BuildLineSeries(item, "Total Available");
                lineLayer.Series.Add(Series);
            }

            CompositeLegend myLegend = new CompositeLegend();

            myLegend.BoundsMeasureType = MeasureType.Percentage;
            myLegend.Bounds = new System.Drawing.Rectangle(15, 90, 60, 6);
            myLegend.LabelStyle.WrapText = true;

            myLegend.LabelStyle.FontSizeBestFit = true;
            myLegend.PE.ElementType = PaintElementType.Gradient;
            myLegend.PE.FillGradientStyle = Infragistics.UltraChart.Shared.Styles.GradientStyle.Circular;
            myLegend.PE.Fill = Color.White;
            myLegend.PE.FillStopColor = Color.White;
            myLegend.Border.CornerRadius = 10;
            myLegend.Border.Thickness = 1;
            myLegend.LabelStyle.FontSizeBestFit = false;
            this.ultraChart2.Legend.Location = LegendLocation.Bottom;

            //add legand to chart
            this.ultraChart2.CompositeChart.Legends.Add(myLegend);

            this.ultraChart2.CompositeChart.Legends[0].ChartLayers.Add(columnLayer);
            this.ultraChart2.CompositeChart.Legends[0].ChartLayers.Add(lineLayer);

            //set chart title
            this.ultraChart2.TitleTop.Text = "Used vs Available";
            this.ultraChart2.TitleTop.HorizontalAlign = StringAlignment.Center;
            ultraChart2.TitleTop.FontSizeBestFit = true;

            //set chart column title
            this.ultraChart2.TitleLeft.Text = "Count";
            this.ultraChart2.TitleLeft.Visible = true;
            this.ultraChart2.TitleLeft.HorizontalAlign = StringAlignment.Center;
            this.ultraChart2.TitleLeft.Margins.Left = 1;
            this.ultraChart2.CompositeChart.ChartLayers.Add(columnLayer);
            this.ultraChart2.CompositeChart.ChartLayers.Add(lineLayer);
        }
        #endregion // Form Load

        #region Create Data
        private List<Data> CreateColumnData()
        {
            var list = new List<Data>();

            list.Add(new Data() { ProductName = "ProductA-Used", Year1 = 1, Year2 = 2, Year3 = 3, Year4 = 4, Year5 = 5, Year6 = 6, Year7 = 7, Year8 = 920, Year9 = 890, Year10 = 10, Year11 = 154, Year12 = 154, Year13 = 0, Year14 = 15, Year15 = 15 });
            list.Add(new Data() { ProductName = "ProductB-Used", Year1 = 2, Year2 = 3, Year3 = 4, Year4 = 15, Year5 = 6, Year6 = 7, Year7 = 8, Year8 = 9, Year9 = 40, Year10 = 11, Year11 = 154, Year12 = 154, Year13 = 0, Year14 = 10, Year15 = 15 });
            list.Add(new Data() { ProductName = "ProductC-Used", Year1 = 3, Year2 = 4, Year3 = 5, Year4 = 6, Year5 = 10, Year6 = 8, Year7 = 9, Year8 = 10, Year9 = 11, Year10 = 12, Year11 = 154, Year12 = 154, Year13 = 7, Year14 = 0, Year15 = 15 });
            list.Add(new Data() { ProductName = "ProductD-Used", Year1 = 4, Year2 = 5, Year3 = 6, Year4 = 7, Year5 = 8, Year6 = 9, Year7 = 10, Year8 = 11, Year9 = 12, Year10 = 13, Year11 = 154, Year12 = 154, Year13 = 50, Year14 = 5, Year15 = 15, Year20 = 250 });
            list.Add(new Data() { ProductName = "ProductE-Used", Year1 = 4, Year2 = 15, Year3 = 6, Year4 = 7, Year5 = 8, Year6 = 9, Year7 = 10, Year8 = 11, Year9 = 12, Year10 = 13, Year11 = 154, Year12 = 154, Year13 = 0, Year14 = 20, Year15 = 15, Year20 = 270 });
            list.Add(new Data() { ProductName = "ProductF-Used", Year1 = 41, Year2 = 15, Year3 = 6, Year4 = 7, Year5 = 8, Year6 = 9, Year7 = 10, Year8 = 11, Year9 = 12, Year10 = 13, Year11 = 154, Year12 = 154, Year13 = 0, Year14 = 50, Year15 = 15, Year20 = 200 });

            return list;
        }

        private List<Data> CreateLineData()
        {
            var list = new List<Data>();

            list.Add(new Data() { ProductName = "Total Available", Year1 = 110, Year2 = 12, Year3 = 622, Year4 = 120, Year5 = 125, Year6 = 16, Year7 = 437, Year8 = 78, Year9 = 0, Year10 = 13, Year11 = 154, Year13 = 0, Year14 = 554, Year15 = 15, Year20 = 400 });

            return list;
        }
        #endregion // Create Data

        #region Build Series
        private NumericSeries BuildColumnSeries(List<Data> list, string ValueColumn, string Label)
        {
            NumericSeries Series = new NumericSeries();
            var propertyInfo = typeof(Data).GetProperty(ValueColumn);

            foreach (var item in list)
            {
                Series.Points.Add(new NumericDataPoint((int)propertyInfo.GetValue(item), item.ProductName, false));
            }

            Series.Label = Label;

            ultraChart2.CompositeChart.Series.Add(Series);
            return Series;
        }

        private NumericSeries BuildLineSeries(Data item, string Label)
        {
            NumericSeries Series = new NumericSeries();
            var propertyInfos = typeof(Data).GetProperties();

            foreach (var propertyInfo in propertyInfos)
            {
                if (propertyInfo.Name != "ProductName")
                {
                    Series.Points.Add(new NumericDataPoint((int)propertyInfo.GetValue(item), item.ProductName, false));
                }
            }

            Series.Label = Label;

            ultraChart2.CompositeChart.Series.Add(Series);

            // This is how to change the series color.
            Series.PEs.Add(new PaintElement(Color.Red));

            return Series;
        }
        #endregion // Build Series

    }
    #endregion // Form1
}
